@@ -1124,7 +1124,7 @@ local function builtin_graveyard_run(conf, _, cmdline, env)
 	 e, i = luxio.readdir(dirp)
 	 if e == 0 then
 	    if not i.d_name:find("^%.") then
-	       log.state(i.d_name)
+	       log.stdout(i.d_name)
 	    end
 	 end
       until not e
