@@ -281,7 +281,8 @@ local function builtin_create_run(config, repo, cmdline, env)
    end
    local owner = cmdline[3] or env["GITANO_USER"]
    log.chat("Setting repository owner to", owner)
-   ok, msg = repo:set_owner(owner, env.GITANO_USER, env.GITANO_ORIG_USER)
+   ok, msg = repo:conf_set_and_save("project.owner", owner,
+				    env.GITANO_USER, env.GITANO_ORIG_USER)
    if not ok then
       log.error(msg)
       return "exit", 1
@@ -603,11 +604,11 @@ local function builtin_destroy_run(config, repo, cmdline, env)
    local token = repo:generate_confirmation("destroy repo " .. repo.name)
    if #cmdline == 2 then
       -- Generate the confirmation token
-      log.state("")
-      log.state("If you are *certain* you wish to destroy this repository")
-      log.state("Then re-run your command with the following confirmation token:")
-      log.state("")
-      log.state("   ", token)
+      log.stdout("")
+      log.stdout("If you are *certain* you wish to destroy this repository")
+      log.stdout("Then re-run your command with the following confirmation token:")
+      log.stdout("")
+      log.stdout("   ", token)
    else
       if cmdline[3] ~= token and cmdline[3] ~= '--force' then
 	 log.error("Confirmation token does not match, refusing to destroy")
@@ -623,15 +624,15 @@ local function builtin_destroy_run(config, repo, cmdline, env)
 	 log.error(msg)
 	 return "exit", 1
       end
-      log.state("Should you need to recover the repository you just destroyed")
-      log.state("then you will need to speak with an admin as soon as possible")
-      log.state("")
-      log.state("When you do, be sure to include the current time (" .. 
+      log.stdout("Should you need to recover the repository you just destroyed")
+      log.stdout("then you will need to speak with an admin as soon as possible")
+      log.stdout("")
+      log.stdout("When you do, be sure to include the current time (" .. 
 	       nowstamp .. ").")
-      log.state("It may also help if you include your token:")
-      log.state("  ", token)
-      log.state("")
-      log.state("Successfully destroyed", repo.name)
+      log.stdout("It may also help if you include your token:")
+      log.stdout("  ", token)
+      log.stdout("")
+      log.stdout("Successfully destroyed", repo.name)
    end
    return "exit", 0
 end
@@ -655,19 +656,11 @@ local function builtin_rename_validate(config, repo, cmdline)
       log.error("Rename takes a repository and a new name for it")
       return false
    end
-   if not repo or repo.is_nascent then
-      log.error("Cannot rename", repo.name, "as it does not exit")
-      return false
-   end
    return true
 end
 
 local function builtin_rename_prep(config, repo, cmdline, context)
    local ctx, action, reason
-   -- Check 0, is the current repo nascent
-   if repo.is_nascent then
-      return "deny", "Cannot rename a repository which does not exist"
-   end
    -- Check 1, read current repo
    ctx = util.deep_copy(context)
    ctx.operation = "read"
@@ -675,27 +668,32 @@ local function builtin_rename_prep(config, repo, cmdline, context)
    if action ~= "allow" then
       return action, reason
    end
-   -- Check 2, rename current repo
+   -- Check 2, is the current repo nascent
+   if repo.is_nascent then
+      return "deny", "Cannot rename a repository which does not exist"
+   end
+   -- Check 3, rename current repo
    ctx = util.deep_copy(context)
    ctx.operation = "renamerepo"
    action, reason = repo:run_lace(ctx)
    if action ~= "allow" then
       return action, reason
    end
-   -- Check 3, create new repo
+   -- Check 4, create new repo
    ctx = util.deep_copy(context)
    local newrepo, msg = repository.find(config, cmdline[3])
    if not newrepo then
       return "deny", msg
    end
-   if not newrepo.is_nascent then
-      return "deny", "Destination location is in use"
-   end
    ctx.operation="createrepo"
    action, reason = newrepo:run_lace(ctx)
    if action ~= "allow" then
       return action, reason
    end
+   -- Check 5, does new repo already exist?
+   if not newrepo.is_nascent then
+      return "deny", "Destination location is in use"
+   end
    -- Okay, we could create, read, and destroy
    -- thus we can rename
    return "allow", "Passed all checks, can rename"
