@@ -13,6 +13,7 @@ local log = require 'gitano.log'
 local config = require 'gitano.config'
 local util = require 'gitano.util'
 local lace = require 'gitano.lace'
+local i18n = require 'gitano.i18n'
 local markdown = require 'gitano.markdown'
 local clod = require 'clod'
 
@@ -41,10 +42,12 @@ function repo_method:load_from_admin_ref(filename)
    local flat_tree = gall.tree.flatten(admintree)
    local entry = flat_tree[filename]
    if not entry then
-      return nil, "Not found: " .. admincommit.sha .. "::" .. filename
+      return nil, i18n.expand("ERROR_GENERIC_CONF_NOT_FOUND",
+			      {sha=admincommit.sha, name=filename})
    end
    if entry.obj.type ~= "blob" then
-      return nil, admincommit.sha .. "::" .. filename .. ": Not a blob"
+      return nil, i18n.expand("ERROR_GENERIC_CONF_NOT_BLOB",
+			      {sha=admincommit.sha, name=filename})
    end
    return entry.obj.content, admincommit.sha .. "::" .. filename
 end
@@ -103,7 +106,7 @@ function repo_method:run_checks()
 	 gall.commit.create(self.git, {
 			      author = admin_name,
 			      committer = admin_name,
-			      message = "Initial admin tree",
+			      message = i18n.expand("INITIAL_ADMIN_TREE"),
 			      tree = tree
 			   })
       if not commit then
@@ -163,8 +166,6 @@ end
 function repo_method:run_lace(context)
    self:populate_context(context)
    config.populate_context(self.config, context)
-   log.ddebug("Running lace for <" .. self.name .. ">, operation <" .. 
-	      (context.operation or "UNKNOWN") .. ">")
    return lace.run(self.lace, context)
 end
 
@@ -225,16 +226,14 @@ function repo_method:check_local_git_files()
       anonexport = true
    end
    if anonexport then
-      log.ddebug("<" .. self.name .. ">: Anonymous read allowed")
       local fh, errmsg = sio.open(self:fs_path() .. "/git-daemon-export-ok", "wc")
 
       if fh then
         fh:close()
       else
-        log.warn("Can't create git-daemon-export file:", errmsg)
+        log.warn(i18n.expand("ERROR_CANNOT_CREATE_EXPORT_FILE", {reason=errmsg}))
       end
    else
-      log.ddebug("<" .. self.name .. ">: Anonymous read not allowed")
       luxio.unlink(self:fs_path() .. "/git-daemon-export-ok")
    end
    -- Now store our description into the description file
@@ -246,11 +245,12 @@ function repo_method:check_local_git_files()
       local ok, errno = luxio.rename(self:fs_path() .. ("/description.%d.new"):format(luxio.getpid()),
 				     self:fs_path() .. "/description")
       if ok ~= 0 then
-	 log.warn("Unable to rename description into place:",
-		  luxio.strerror(errno))
+	 log.warn(i18n.expand("ERROR_UNABLE_TO_RENAME_INTO_PLACE",
+			      {what="description", reason=luxio.strerror(errno)}))
       end
    else
-      log.warn("Unable to write description:", tostring(err))
+      log.warn(i18n.expand("ERROR_UNABLE_TO_WRITE_FILE",
+			   {what="description", reason=tostring(err)}))
    end
    -- And now our readme if we have one
    if self.readme_html then
@@ -261,11 +261,12 @@ function repo_method:check_local_git_files()
 	 local ok, errno = luxio.rename(self:fs_path() .. ("/README.html.%d.new"):format(luxio.getpid()),
 					self:fs_path() .. "/README.html")
 	 if ok ~= 0 then
-	    log.warn("Unable to rename README.html into place:",
-		     luxio.strerror(errno))
+	    log.warn(i18n.expand("ERROR_UNABLE_TO_RENAME_INTO_PLACE",
+			      {what="README.html", reason=luxio.strerror(errno)}))
 	 end
       else
-	 log.warn("Unable to write README.html out:", tostring(err))
+	 log.warn(i18n.expand("ERROR_UNABLE_TO_WRITE_FILE",
+			      {what="README.html", reason=tostring(err)}))
       end
    else
       -- Remove any readme (and ignore errors)
@@ -317,18 +318,19 @@ function repo_method:check_local_git_files()
       local ok, errno = luxio.rename(self:fs_path() .. ("/cgitrc.%d.new"):format(luxio.getpid()),
 				     self:fs_path() .. "/cgitrc")
       if ok ~= 0 then
-	 log.warn("Unable to rename cgitrc into place:",
-		  luxio.strerror(errno))
+	 log.warn(i18n.expand("ERROR_UNABLE_TO_RENAME_INTO_PLACE",
+			      {what="cgitrc", reason=luxio.strerror(errno)}))
       end
    else
-      log.warn("Unable to write cgitrc out:", tostring(err))
+      log.warn(i18n.expand("ERROR_UNABLE_TO_WRITE_FILE",
+			   {what="cgitrc", reason=tostring(err)}))
    end
 
    return true
 end
 
 function repo_method:check_and_upgrade_hooks()
-   log.ddebug("Checking hooks for", self.name)
+   log.ddebug(i18n.expand("DEBUG_CHECKING_HOOKS", {name=self.name}))
    for _, hook in ipairs {"pre-receive", "update", "post-receive"} do
       local hookfile = ("%s/hooks/%s"):format(self:fs_path(), hook)
       local hooktarget = ("%s/gitano-%s-hook"):format(config.lib_bin_path(),
@@ -352,7 +354,7 @@ end
 
 function repo_method:validate_admin_sha(sha)
    if sha == string.rep("0", 40) then
-      return nil, "A deleted admin ref cannot be loaded"
+      return nil, i18n.expand("ERROR_DELETED_ADMIN_REF")
    end
    local commit = self.git:get(sha)
    local tree = gall.tree.flatten(commit.content.tree.content)
@@ -362,7 +364,7 @@ function repo_method:validate_admin_sha(sha)
    end
 
    if not is_blob(tree["project.conf"]) then
-      return nil, "Unable to find project.conf in repository"
+      return nil, i18n.expand("ERROR_NO_PROJECT_CONF")
    end
 
    local conf_text = tree["project.conf"].obj.content
@@ -409,7 +411,8 @@ function repo_method:validate_admin_sha(sha)
 	 ret.readme_html = res
 	 ret.readme_mdwn = mdwn_src
       else
-	 log.warn("<" .. self.name .. "> Markdown:", res)
+	 log.warn(i18n.expand("WARN_MARKDOWN_ERROR", {name=self.name,
+						      reason=res}))
       end
    end
 
@@ -427,11 +430,6 @@ end
 function repo_method:populate_context(context)
    util.add_splitable(context, "repository", self.name,
 		      "/", "dirname", "basename")
-   if self.is_nascent then
-      context["owner"] = "gitano/nobody"
-   else
-      context["owner"] = self:conf_get "project.owner"
-   end
    context["_repo"] = self
    if not self.is_nascent then
       local lists_to_add = {}
@@ -455,11 +453,11 @@ end
 
 function repo_method:realise()
    if not self.is_nascent then
-      return false, "Cannot realise a non-nascent repository."
+      return false, i18n.expand("ERROR_REALISE_NON_NASCENT")
    end
    
    if not util.mkdir_p(util.dirname(self:fs_path())) then
-      return false, "Cannot prepare path leading to repository."
+      return false, i18n.expand("ERROR_CANNOT_PREP_PATH")
    end
 
    local r, msg = gall.repository.create(self:fs_path())
@@ -474,42 +472,29 @@ function repo_method:realise()
    return self:run_checks()
 end
 
-function repo_method:set_owner(newowner, author, committer)
-   local oldowner = self:conf_get "project.owner"
-   self:conf_set("project.owner", newowner)
-   local ok, msg = self:save_admin("Setting owner to " .. newowner,
+function repo_method:conf_set_and_save(conf, newvalue, author, committer)
+   local oldvalue = self:conf_get(conf)
+   self:conf_set(conf, newvalue)
+   local ok, msg = self:save_admin("Setting " .. conf .. " to " .. newvalue,
 				   author, committer)
    if not ok then
-      self:conf_set("project.owner", oldowner)
+      self:conf_set(conf, oldvalue)
       return nil, msg
    end
-   log.state("<" .. self.name .. "> Set owner to <" .. newowner .. ">")
-   return true
-end
-
-function repo_method:set_description(newdesc, author, committer)
-   local olddesc = self:conf_get "project.description"
-   self:conf_set("project.description", newdesc)
-   local ok, msg = self:save_admin("Changing description\n\n" .. newdesc,
-				   author, committer)
-   if not ok then
-      self:conf_set("project.description", olddesc)
-      return nil, msg
-   end
-   log.state("<" .. self.name .. "> Changed description")
+   log.state("<" .. self.name .. "> Set " .. conf .. " to <" .. newvalue .. ">")
    return true
 end
 
 function repo_method:set_readme(newreadme, author, committer)
    local oldreadme = self.readme_mdwn
    self.readme_mdwn = newreadme
-   local ok, msg = self:save_admin("Changing readme",
+   local ok, msg = self:save_admin(i18n.expand("MSG_CHANGING_README"),
 				   author, committer)
    if not ok then
       self.readme_mdwn = oldreadme
       return nil, msg
    end
-   log.state("<" .. self.name .. "> Changed readme")
+   log.state(i18n.expand("MSG_CHANGED_README", {name=self.name}))
    return true
 end
 
@@ -519,13 +504,13 @@ function repo_method:set_head(newhead, author, committer)
    end
    local oldhead = self:conf_get "project.head"
    self:conf_set("project.head", newhead)
-   local ok, msg = self:save_admin("Changing project head",
+   local ok, msg = self:save_admin(i18n.expand("MSG_CHANGING_HEAD"),
 				   author, committer)
    if not ok then
       self:conf_set("project.head", oldhead)
       return nil, msg
    end
-   log.state("<" .. self.name .. "> Set head to <" .. newhead .. ">")
+   log.state(i18n.expand("MSG_CHANGED_HEAD", {name=self.name, head=newhead}))
    return true
 end
 
@@ -541,7 +526,7 @@ function repo_method:generate_confirmation(notes)
    end
    table.sort(refnames)
    local str = notes or ""
-   log.debug("Calculating confirmation token for", self.name)
+   log.debug(i18n.expand("DEBUG_CALCULATE_TOKEN", {name=self.name}))
    for i = 1, #refnames do
       local ref, sha = refnames[i], refs[refnames[i]]
       log.ddebug(ref, "@", sha)
@@ -556,15 +541,15 @@ function repo_method:destroy_self(call_me)
    -- Is our destination location in the graveyard free?
    local ok, err = luxio.stat(graveyard_path)
    if ok == 0 then
-      return false, "Our grave site is already occupied! Cannot destroy."
+      return false, i18n.expand("ERROR_GRAVESITE_OCCUPIED")
    end
    if err ~= luxio.ENOENT then
-      return false, "Could not check grave site for occupancy. Cannot destroy."
+      return false, i18n.expand("ERROR_GRAVESITE_UNCHECKED")
    end
    -- Grave site is not occupied, attempt to rename ourselves
    local ok, err = luxio.rename(self:fs_path(), graveyard_path)
    if ok ~= 0 then
-      return false, "Cannot destroy. rename() returned " .. luxio.strerror(err)
+      return false, i18n.expand("ERROR_CANNOT_DESTROY_RENAME", {reason=luxio.strerror(err)})
    end
    -- Successfully renamed ourselves, we're destroyed.
    return true
@@ -583,12 +568,12 @@ function repo_method:rename_to(somename)
    local newpath = self.fs_path({name=somename,config=self.config})
 
    if not util.mkdir_p(util.dirname(newpath)) then
-      return false, "Cannot prepare path leading to repository."
+      return false, i18n.expand("ERROR_CANNOT_PREP_PATH")
    end
 
    local ok, err = luxio.rename(self:fs_path(), newpath)
    if ok ~= 0 then
-      return false, "Unable to rename repository: " .. luxio.strerror(err)
+      return false, i18n.expand("ERROR_CANNOT_RENAME_REPO", {reason=luxio.strerror(err)})
    end
    return true
 end
@@ -597,7 +582,7 @@ function repo_method:copy_to(target)
    local ok, err
 
    if not target.is_nascent then
-      return false, "Target repository is not Nascent"
+      return false, i18n.expand("ERROR_TARGET_NOT_NASCENT")
    end
 
    local newpath = target:fs_path()
@@ -605,7 +590,7 @@ function repo_method:copy_to(target)
    local temp_path = newpath .. ".in_progress"
 
    if not util.mkdir_p(util.dirname(temp_path)) then
-      return false, "Cannot prepare path leading to repository."
+      return false, i18n.expand("ERROR_CANNOT_PREP_PATH")
    end
 
    -- attempt to create the target directory, so we can detect
@@ -613,9 +598,11 @@ function repo_method:copy_to(target)
    -- the target directory
    ok, err = luxio.mkdir(temp_path, sio.tomode'0755')
    if ok ~= 0 then
-      log.error("Failed to copy repository", self:fs_path(),
-                "to", newpath .. ":", "Copy already in progress")
-      return false, "Copy already in progress"
+      log.error(i18n.expand("ERROR_FAILED_TO_COPY",
+			    {path=self:fs_path(),
+			     newpath=newpath,
+			     reason=i18n.expand("ERROR_COPY_IN_PROGRESS")}))
+      return false, i18n.expand("ERROR_COPY_IN_PROGRESS")
    end
 
    local from = self:fs_path()
@@ -626,9 +613,10 @@ function repo_method:copy_to(target)
    -- copy non-objects parts of the git repository
    ok, err = util.copy_dir(from, temp_path, nil, filter)
    if not ok then
-      log.error("Failed to copy repository", from, "to", temp_path, err)
+      log.error(i18n.expand("ERROR_FAILED_TO_COPY",
+			    {path=from, newpath=temp_path, reason=err}))
       util.rm_rf(temp_path)
-      return false, "Failed to copy repository"
+      return false, i18n.expand("ERROR_FAILED_COPYING_CONTENT")
    end
    -- Hardlink the objects tree
    local cbs = util.deep_copy(util.copy_dir_copy_callbacks)
@@ -637,18 +625,20 @@ function repo_method:copy_to(target)
                            util.path_join(temp_path, 'objects'),
                            cbs)
    if not ok then
-      log.error("Failed to hardlink objects of", from, "to", temp_path, err)
+      log.error(i18n.expand("ERROR_FAILED_HARDLINKING",
+			    {path=from, newpath=temp_path, reason=err}))
       util.rm_rf(temp_path)
-      return ok, "Failed to copy repository"
+      return ok, i18n.expand("ERROR_FAILED_COPYING_CONTENT")
    end
 
    -- rename into place
    ok, err = luxio.rename(temp_path, newpath)
    if ok ~= 0 then
-      log.error("Failed to rename repository", temp_path, "to",
-                newpath, luxio.strerror(err))
+      log.error(i18n.expand("ERROR_FAILED_RENAME_REPOSITORY",
+			    {from=temp_path, to=newpath,
+			     reason=luxio.strerr(err)}))
       util.rm_rf(temp_path)
-      return false, "Failed to copy repository"
+      return false, i18n.expand("ERROR_FAILED_COPYING_CONTENT")
    end
 
    return true
@@ -660,7 +650,7 @@ function repo_method:update_modified_date(shas)
    local modfile = dirpath .. "/last-modified"
 
    if not util.mkdir_p(dirpath) then
-      return false, "Cannot prepare path leading to info file."
+      return false, i18n.expand("ERROR_CANNOT_PREP_PATH_INFO")
    end
 
    local last_mod_mtime = 0
@@ -698,7 +688,7 @@ function repo_method:update_modified_date(shas)
    end
    f = io.open(modfile, "w")
    if not f then
-      return false, "Could not open info/web/last-modified for writing"
+      return false, i18n.expand("ERROR_CANNOT_WRITE_LAST_MODIFIED")
    end
    f:write(("%d %s\n"):format(last_mod_mtime, last_mod_offset))
    f:close()
@@ -743,7 +733,7 @@ function repo_method:save_admin(reason, author, committer)
       gall.commit.create(self.git, {
 			   author = author,
 			   committer = committer,
-			   message = reason or "Updated admin tree",
+			   message = reason or i18n.expand("MSG_UPDATED_ADMIN_TREE"),
 			   tree = tree,
 			   parents = { curcommit }
 			})
@@ -821,7 +811,7 @@ local function check_repodir(dirname, repo)
 	 end
       until not e
       if next(needed) then
-	 return nil, next(needed) .. " missing"
+	 return nil, i18n.expand("ERROR_SOMETHING_MISSING", {what=next(needed)})
       end
    end
 
@@ -835,7 +825,6 @@ local function find_repository(config, reponame)
    --
    -- If the repository exists, then it is examined and brought up-to-date
    -- with any global config changes before being returned.
-   log.ddebug("find_repository", reponame)
    -- Inject a leading '/'
    reponame = "/" .. reponame
    -- Remove any spaces, tabs, newlines or nulls
@@ -850,7 +839,6 @@ local function find_repository(config, reponame)
    if reponame:match(".%.git$") then
       reponame = reponame:match("^(.+)%.git$")
    end
-   log.ddebug("find_repository", "cleaned", reponame)
 
    -- Construct the repo
    local repo = setmetatable({config = config, name = reponame}, repo_meta)
